###################################################
### chunk number 1: preliminaries
###################################################
library("deSolve")
options(prompt = "R> ")
options(width=70)


###################################################
### chunk number 2: the_Rmodel
###################################################
model <- function(t, Y, parameters) {
  with (as.list(parameters),{

    dy1 = -k1*Y[1] + k2*Y[2]*Y[3]
    dy3 = k3*Y[2]*Y[2]
    dy2 = -dy1 - dy3

    list(c(dy1, dy2, dy3))
  })
}


###################################################
### chunk number 3: Jacobian_in_R
###################################################
jac <- function (t, Y, parameters) {
  with (as.list(parameters),{

    PD[1,1] = -k1
    PD[1,2] = k2*Y[3]
    PD[1,3] = k2*Y[2]
    PD[2,1] = k1
    PD[2,3] = -PD[1,3]
    PD[3,2] = k3*Y[2]
    PD[2,2] = -PD[1,2] - PD[3,2]

    return(PD)
  })
}


###################################################
### chunk number 4: Run_Rmodel
###################################################
parms <- c(k1 = 0.04, k2 = 1e4, k3=3e7)
Y     <- c(1.0, 0.0, 0.0)
times <- c(0, 0.4*10^(0:11) )
PD    <- matrix(nrow = 3, ncol = 3, data = 0)
out   <- ode(Y, times, model, parms = parms, jacfunc = jac)


###################################################
### chunk number 5: compile_DLLmodel_F eval=FALSE
###################################################
##   system("R CMD SHLIB mymod.f")


###################################################
### chunk number 6: compile_DLLmodel_C eval=FALSE
###################################################
##   system("R CMD SHLIB mymod.c")


###################################################
### chunk number 7: run_DLLmodel eval=FALSE
###################################################
## dyn.load("mymod.dll")
## 
## parms <- c(k1 = 0.04, k2 = 1e4, k3=3e7)
## Y     <- c(y1 = 1.0, y2 = 0.0, y3 = 0.0)
## times <- c(0, 0.4*10^(0:11) )
## 
## out <- ode(Y, times, func = "derivs", parms = parms,
##            jacfunc = "jac", dllname = "mymod",
##            initfunc = "initmod", nout = 1, outnames = "Sum")


###################################################
### chunk number 8: 
###################################################
## Parameter values and initial conditions
Parms <- c(0.182, 4.0, 4.0, 0.08, 0.04, 0.74, 0.05, 0.15, 0.32,
        16.17, 281.48, 13.3, 16.17, 5.487, 153.8, 0.04321671,
        0.4027255, 1000, 0.02, 1.0, 3.8)

yini <- c( AI=21, AAM=0, AT=0, AF=0, AL=0, CLT=0,  AM=0  )

## the rate of change
DLLfunc(y = yini, dllname = "deSolve", func = "derivsccl4",
        initfunc = "initccl4", parms = Parms, times = 1,
        nout = 3, outnames = c("DOSE", "MASS", "CP")  )


###################################################
### chunk number 9: 
###################################################
pars <- c(K = 1, ka   = 1e6, r    = 1)

## Initial conc; D is in equilibrium with A,B
y     <- c(A = 2, B = 3, D = 2*3/pars["K"])

## Initial rate of change
dy  <- c(dA = 0, dB = 0, dD = 0)

## production increases with time
prod <- matrix(nc=2,data=c(seq(0,100,by=10),seq(0.1,0.5,len=11)))

DLLres(y=y,dy=dy,times=5,res="chemres",
          dllname="deSolve", initfunc="initparms",
          initforc="initforcs", parms=pars, forcings=prod,
          nout=2, outnames=c("CONC","Prod"))



###################################################
### chunk number 10: 
###################################################
Flux <- matrix(ncol=2,byrow=TRUE,data=c(
  1, 0.654, 11, 0.167,   21, 0.060, 41, 0.070, 73,0.277, 83,0.186,
  93,0.140,103, 0.255,  113, 0.231,123, 0.309,133,1.127,143,1.923,
  153,1.091,163,1.001,  173, 1.691,183, 1.404,194,1.226,204,0.767,
  214, 0.893,224,0.737, 234,0.772,244, 0.726,254,0.624,264,0.439,
  274,0.168,284 ,0.280, 294,0.202,304, 0.193,315,0.286,325,0.599,
  335, 1.889,345, 0.996,355,0.681,365,1.135))
head(Flux)


###################################################
### chunk number 11: 
###################################################
parms <- 0.01


###################################################
### chunk number 12: 
###################################################
meanDepo <- mean(approx(Flux[,1],Flux[,2], xout=seq(1,365,by=1))$y)
Yini <- c(y=meanDepo/parms)


###################################################
### chunk number 13: 
###################################################
times <- 1:365
out <- as.data.frame( ode(y=Yini, times, func = "scocder",
    parms = parms, dllname = "deSolve",
    initforc="scocforc",  forcings=Flux,
    initfunc = "scocpar", nout = 2,
    outnames = c("Mineralisation","Depo")))
head(out)


###################################################
### chunk number 14: 
###################################################
fcontrol <- list(method="constant")
out2 <- as.data.frame( ode(y=Yini, times, func = "scocder",
    parms = parms, dllname = "deSolve",
    initforc="scocforc",  forcings=Flux, fcontrol=fcontrol,
    initfunc = "scocpar", nout = 2,
    outnames = c("Mineralisation","Depo")))


###################################################
### chunk number 15: scoc
###################################################
par (mfrow=c(1,2))
plot(out$time, out$Depo, type="l", col="red",
  xlab="days", ylab="mmol C/m2/ d", main="method='linear'")
lines(out$time, out$Mineralisation, lwd=2, col="blue")
legend("topleft",lwd=1:2,col=c("red","blue"), c("Flux","Mineralisation"))

plot(out2$time, out2$Depo, type="l", col="red",
  xlab="days", ylab="mmol C/m2/ d", main="method='constant'")
lines(out2$time, out2$Mineralisation, lwd=2, col="blue")


###################################################
### chunk number 16: figscoc
###################################################
par (mfrow=c(1,2))
plot(out$time, out$Depo, type="l", col="red",
  xlab="days", ylab="mmol C/m2/ d", main="method='linear'")
lines(out$time, out$Mineralisation, lwd=2, col="blue")
legend("topleft",lwd=1:2,col=c("red","blue"), c("Flux","Mineralisation"))

plot(out2$time, out2$Depo, type="l", col="red",
  xlab="days", ylab="mmol C/m2/ d", main="method='constant'")
lines(out2$time, out2$Mineralisation, lwd=2, col="blue")


###################################################
### chunk number 17: 
###################################################
SPCmod <- function(t, x, parms, input)  {
  with(as.list(c(parms, x)), {
    import <- input(t)
    dS <- import - b*S*P + g*C    # substrate
    dP <- c*S*P  - d*C*P          # producer
    dC <- e*P*C  - f*C            # consumer
    res <- c(dS, dP, dC)
    list(res, signal = import)
  })
}

## The parameters
parms  <- c(b = 0.1, c = 0.1, d = 0.1, e = 0.1, f = 0.1, g = 0.0)

## vector of timesteps
times  <- seq(0, 100, by=0.1)

## external signal with several rectangle impulses
signal <- as.data.frame(list(times = times,
                 import = rep(0, length(times))))

signal$import <- ifelse((trunc(signal$times) %% 2 == 0), 0, 1)
sigimp <- approxfun(signal$times, signal$import, rule = 2)

## Start values for steady state
xstart <- c(S = 1, P = 1, C = 1)

## Solve model
print (system.time(
out <- ode(y = xstart,times = times,
           func = SPCmod, parms, input = sigimp)
))


###################################################
### chunk number 18: lv
###################################################
plot(out, type="l")


###################################################
### chunk number 19: figlv
###################################################
plot(out, type="l")


###################################################
### chunk number 20: 
###################################################
eventdata <- data.frame(var=rep("C",10),time=seq(10,100,10),value=rep(0.5,10),
  method=rep("multiply",10))
eventdata


