###################################################
### chunk number 1: preliminaries
###################################################
library("deSolve")
options(prompt = "> ")
options(width=70)


###################################################
### chunk number 2: 
###################################################
parameters <- c(a = -8/3,
                b = -10,
                c =  28)


###################################################
### chunk number 3: 
###################################################
state     <- c(X = 1,
               Y = 1,
               Z = 1)


###################################################
### chunk number 4: 
###################################################
Lorenz<-function(t, state, parameters) {
  with(as.list(c(state, parameters)),{
   # rate of change
    dX <- a*X + Y*Z
    dY <- b * (Y-Z)
    dZ <- -X*Y + c*Y - Z

    # return the rate of change
    list(c(dX, dY, dZ))
  })   # end with(as.list...
}


###################################################
### chunk number 5: 
###################################################
times     <-seq(0,100,by=0.01)


###################################################
### chunk number 6: 
###################################################
require(deSolve)
out <- as.data.frame(ode(y=state,times=times,func=Lorenz,parms=parameters))
head(out)


###################################################
### chunk number 7: ode
###################################################
par(mfrow=c(2,2), oma=c(0,0,3,0))
plot (times,out$X ,type="l",main="X", xlab="time", ylab="-")
plot (times,out$Y ,type="l",main="Y", xlab="time", ylab="-")
plot (out$X,out$Y, pch=".")
plot (out$X,out$Z, pch=".")

mtext(outer=TRUE,side=3,"Lorenz model",cex=1.5)


###################################################
### chunk number 8: figode
###################################################
par(mfrow=c(2,2), oma=c(0,0,3,0))
plot (times,out$X ,type="l",main="X", xlab="time", ylab="-")
plot (times,out$Y ,type="l",main="Y", xlab="time", ylab="-")
plot (out$X,out$Y, pch=".")
plot (out$X,out$Z, pch=".")

mtext(outer=TRUE,side=3,"Lorenz model",cex=1.5)


###################################################
### chunk number 9: 
###################################################
print(system.time(out1 <- rk4   (state, times, Lorenz, parameters)))
print(system.time(out2 <- lsode (state, times, Lorenz, parameters)))
print(system.time(out  <- lsoda (state, times, Lorenz, parameters)))
print(system.time(out  <- lsodes(state, times, Lorenz, parameters)))
print(system.time(out  <- daspk (state, times, Lorenz, parameters)))
print(system.time(out  <- vode  (state, times, Lorenz, parameters)))


###################################################
### chunk number 10: 
###################################################
diagnostics(out1)
diagnostics(out2)


###################################################
### chunk number 11: 
###################################################
Aphid <- function(t, APHIDS, parameters) {
  deltax     <- c (0.5, rep(1, numboxes - 1), 0.5)
  Flux       <- -D * diff(c(0, APHIDS, 0)) / deltax
  dAPHIDS    <- -diff(Flux) / delx + APHIDS * r

  # the return value
  list(dAPHIDS )
} # end


###################################################
### chunk number 12: 
###################################################
D         <- 0.3    # m2/day  diffusion rate
r         <- 0.01   # /day    net growth rate
delx      <- 1      # m       thickness of boxes
numboxes  <- 60

# distance of boxes on plant, m, 1 m intervals
Distance  <- seq(from = 0.5, by = delx, length.out = numboxes)


###################################################
### chunk number 13: 
###################################################
# Initial conditions:  # ind/m2
APHIDS        <- rep(0, times = numboxes)
APHIDS[30:31] <- 1
state         <- c(APHIDS = APHIDS)      # initialise state variables


###################################################
### chunk number 14: 
###################################################
times <-seq(0, 200, by = 1)
print(system.time(
  out <- ode.band(state, times, Aphid, parms = 0, nspec = 1)
))


###################################################
### chunk number 15: 
###################################################
head(out[,1:5])
DENSITY <- out[,2:(numboxes  +1)]


###################################################
### chunk number 16: aphid
###################################################
filled.contour(x = times, y = Distance, DENSITY, color = topo.colors,
               xlab = "time, days", ylab = "Distance on plant, m",
               main = "Aphid density on a row of plants")


###################################################
### chunk number 17: aphidfig
###################################################
filled.contour(x = times, y = Distance, DENSITY, color = topo.colors,
               xlab = "time, days", ylab = "Distance on plant, m",
               main = "Aphid density on a row of plants")


###################################################
### chunk number 18: 
###################################################
daefun<-function(t,y,dy,parameters) {
  res1  <- dy[1] + y[1] - y[2]
  res2  <- y[2]*y[1] - t

  list(c(res1, res2))
}
require(deSolve)

yini  <- c(1, 0)
dyini <- c(1, 0)
times <- seq(0, 10, 0.1)

## solver
print(system.time(out <- daspk(y=yini,dy=dyini,times=times,res=daefun,parms=0)))


###################################################
### chunk number 19: dae
###################################################
matplot(out[,1], out[,2:3], type = "l", lwd = 2,
        main = "dae", xlab = "time", ylab = "y")


###################################################
### chunk number 20: figdae
###################################################
matplot(out[,1], out[,2:3], type = "l", lwd = 2,
        main = "dae", xlab = "time", ylab = "y")


###################################################
### chunk number 21: 
###################################################
ZODE2 <- function(Time, State, Pars) {
  with(as.list(State), {
       df <- 1i * f
       dg <- -1i*g*g*f
       return(list(c(df, dg)))
  })
}

yini    <- c(f = 1+0i, g = 1/2.1+0i)
times   <- seq(0, 2*pi, length = 100)

out     <- zvode(func = ZODE2, y = yini, parms = NULL, times = times,
  atol = 1e-10, rtol = 1e-10)


###################################################
### chunk number 22: 
###################################################
analytical  <- cbind(f = exp(1i*times), g = 1/(exp(1i*times)+1.1))
tail(cbind(out[,2], analytical[,1]))


###################################################
### chunk number 23: 
###################################################
f1 <- function  (t, y, parms) {
  ydot <- vector(len = 5)

  ydot[1] <-  0.1*y[1] -0.2*y[2]
  ydot[2] <- -0.3*y[1] +0.1*y[2] -0.2*y[3]
  ydot[3] <-           -0.3*y[2] +0.1*y[3] -0.2*y[4]
  ydot[4] <-                     -0.3*y[3] +0.1*y[4] -0.2*y[5]
  ydot[5] <-                               -0.3*y[4] +0.1*y[5]

  return(list(ydot))
}


###################################################
### chunk number 24: 
###################################################
yini  <- 1:5
times <- 1:20


###################################################
### chunk number 25: 
###################################################
out   <- lsode(yini, times, f1, parms = 0, jactype = "fullint")


###################################################
### chunk number 26: 
###################################################
fulljac <- function  (t, y, parms) {
  jac <- matrix(nrow = 5, ncol = 5, byrow = TRUE,
                data = c(0.1, -0.2,  0  ,  0  ,  0  ,
                        -0.3,  0.1, -0.2,  0  ,  0  ,
                         0  , -0.3,  0.1, -0.2,  0  ,
                         0  ,  0  , -0.3,  0.1, -0.2,
                         0  ,  0  ,  0  , -0.3,  0.1)    )
  return(jac)
}


###################################################
### chunk number 27: 
###################################################
out2  <- lsode(yini, times, f1, parms = 0, jactype = "fullusr",
               jacfunc = fulljac)


###################################################
### chunk number 28: 
###################################################
out3  <- lsode(yini, times, f1, parms = 0, jactype = "bandint",
                              bandup = 1, banddown = 1)


###################################################
### chunk number 29: 
###################################################
bandjac <- function  (t, y, parms) {
  jac <- matrix(nrow = 3, ncol = 5, byrow = TRUE,
                data = c( 0  , -0.2, -0.2, -0.2, -0.2,
                          0.1,  0.1,  0.1,  0.1,  0.1,
                         -0.3, -0.3, -0.3, -0.3,    0)    )
  return(jac)
}


###################################################
### chunk number 30: 
###################################################
out4  <- lsode(yini, times, f1, parms = 0, jactype = "bandusr",
               jacfunc = bandjac, bandup = 1, banddown = 1)


###################################################
### chunk number 31: 
###################################################
out5  <- lsode(yini, times, f1, parms = 0, mf = 10)


###################################################
### chunk number 32: 
###################################################
eventmod <- function(t, var, parms) {
  list(dvar = -0.1*var)
}

yini <- c(v1 = 1, v2 = 2)
times <- seq(0, 10, by = 0.1)


###################################################
### chunk number 33: 
###################################################
eventdat <- data.frame(var = c("v1", "v2", "v2", "v1"), time = c(1, 1, 5, 9),
  value = c(1, 2, 3, 4), method = c("add", "mult", "rep", "add"))

eventdat


###################################################
### chunk number 34: 
###################################################
out <- ode(func = eventmod, y = yini, times = times, parms = NULL,
  events = list(data = eventdat))


###################################################
### chunk number 35: event1
###################################################
plot(out, type = "l", lwd = 2)


###################################################
### chunk number 36: figevent1
###################################################
plot(out, type = "l", lwd = 2)


###################################################
### chunk number 37: 
###################################################
ballode<- function(t, y, parms) {
  dy1 <- y[2]
  dy2 <- -9.8
  list(c(dy1, dy2))
}


###################################################
### chunk number 38: 
###################################################
root <- function(t, y, parms) y[1]


###################################################
### chunk number 39: 
###################################################
event <- function(t, y, parms) {
 y[1]<- 0
 y[2]<- -0.9 * y[2]
 return(y)
}


###################################################
### chunk number 40: 
###################################################
yini  <- c(height = 0, v = 20)
times <- seq(from = 0, to = 20, by = 0.01)

out   <- lsode(times = times, y = yini, func = ballode, parms = NULL,
  events = list(func = event, root = TRUE), rootfun = root)


###################################################
### chunk number 41: event2
###################################################
plot(out, which = "height", type = "l",lwd = 2,
  main = "bouncing ball", ylab = "height")


###################################################
### chunk number 42: figevent2
###################################################
plot(out, which = "height", type = "l",lwd = 2,
  main = "bouncing ball", ylab = "height")


###################################################
### chunk number 43: 
###################################################
require(deSolve)

#-----------------------------
# the derivative function
#-----------------------------
derivs <- function(t, y, parms) {
  if (t < 0)
    lag <- 19
  else
    lag <- lagvalue(t - 0.74)

  dy <- r * y* (1 - lag/m)
  list(dy, dy = dy)
}

#-----------------------------
# parameters
#-----------------------------

r <- 3.5; m <- 19

#-----------------------------
# initial values and times
#-----------------------------

yinit <- c(y = 19.001)
times <- seq(-0.74, 40, by = 0.01)

#-----------------------------
# solve the model
#-----------------------------

yout <- dede(y = yinit, times = times, func = derivs,
             parms = NULL, atol = 1e-10)


###################################################
### chunk number 44: dde
###################################################
plot(yout, which = 1, type = "l", lwd = 2, main = "Lemming model", mfrow = c(1,2))
plot(yout[,2], yout[,3], xlab = "y", ylab = "dy", type = "l", lwd = 2)


###################################################
### chunk number 45: figdde
###################################################
plot(yout, which = 1, type = "l", lwd = 2, main = "Lemming model", mfrow = c(1,2))
plot(yout[,2], yout[,3], xlab = "y", ylab = "dy", type = "l", lwd = 2)


###################################################
### chunk number 46: 
###################################################
SPCmod <- function(t, x, parms)  {
  with(as.list(c(parms, x)), {
    dP <- c*P   - d*C*P      # producer
    dC <- e*P*C - f*C        # consumer
    res <- c(dP, dC)
    list(res)
  })
}


###################################################
### chunk number 47: 
###################################################
parms  <- c(c = 5, d = 0.1, e = 0.1, f = 0.1)


###################################################
### chunk number 48: 
###################################################
xstart <- c(P = 0.5, C = 1)
times  <- seq(0, 190, 0.1)

out <- as.data.frame(ode(y = xstart, times = times,
  func = SPCmod, parms = parms))
tail(out)


###################################################
### chunk number 49: 
###################################################
out <- as.data.frame(ode(y = xstart,times = times, func = SPCmod,
                         parms = parms, atol = 0))
matplot(out[,1], out[,2:3], type = "l")


